package sun.util.calendar;

import java.util.TimeZone;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.ArrayList;
import java.io.IOException;

public class LocalGregorianCalendar extends BaseCalendar
{
    private String name;
    private Era[] eras;
    
    static LocalGregorianCalendar getLocalGregorianCalendar(String str) {
        Properties calendarProperties;
        try {
            calendarProperties = CalendarSystem.getCalendarProperties();
        }
        catch (IOException | IllegalArgumentException cause) {
            throw new InternalError(cause);
        }
        String property = calendarProperties.getProperty("calendar." + str + ".eras");
        if (property == null) {
            return null;
        }
        ArrayList<Era> list = new ArrayList();
        StringTokenizer stringTokenizer = new StringTokenizer(property, ";");
        while (stringTokenizer.hasMoreTokens()) {
            StringTokenizer o = new StringTokenizer(stringTokenizer.nextToken().trim(), ",");
            String s = null;
            boolean n = true;
            long n2 = 0L;
            String s2 = null;
            while (o.hasMoreTokens()) {
                String nextToken = o.nextToken();
                int index = nextToken.indexOf(61);
                if (index == -1) {
                    return null;
                }
                String substring = nextToken.substring(0, index);
                String substring2 = nextToken.substring(index + 1);
                if ("name".equals(substring)) {
                    s = substring2;
                }
                else if ("since".equals(substring)) {
                    if (substring2.endsWith("u")) {
                        n = false;
                        n2 = Long.parseLong(substring2.substring(0, substring2.length() - 1));
                    }
                    else {
                        n2 = Long.parseLong(substring2);
                    }
                }
                else if ("abbr".equals(substring)) {
                    s2 = substring2;
                }
                else {
                    throw new RuntimeException("Unknown key word: " + substring);
                }
            }
            list.add(new Era(s, s2, n2, (boolean)n));
        }
        Era[] array = new Era[list.size()];
        list.toArray(array);
        return new LocalGregorianCalendar(str, array);
    }
    
    private LocalGregorianCalendar(String name, Era[] eras) {
        this.name = name;
        this.setEras(this.eras = eras);
    }
    
    @Override
    public String getName() {
        return this.name;
    }
    
    @Override
    public LocalGregorianCalendar.Date getCalendarDate() {
        return this.getCalendarDate(System.currentTimeMillis(), this.newCalendarDate());
    }
    
    @Override
    public LocalGregorianCalendar.Date getCalendarDate(long n) {
        return this.getCalendarDate(n, this.newCalendarDate());
    }
    
    @Override
    public LocalGregorianCalendar.Date getCalendarDate(long n, TimeZone timeZone) {
        return this.getCalendarDate(n, this.newCalendarDate(timeZone));
    }
    
    @Override
    public LocalGregorianCalendar.Date getCalendarDate(long n, CalendarDate calendarDate) {
        LocalGregorianCalendar.Date date = (LocalGregorianCalendar.Date)super.getCalendarDate(n, calendarDate);
        return this.adjustYear(date, n, date.getZoneOffset());
    }
    
    private LocalGregorianCalendar.Date adjustYear(LocalGregorianCalendar.Date date, long n, int n2) {
        int i = this.eras.length - 1;
        while (i >= 0) {
            Era localEra = this.eras[i];
            long since = localEra.getSince(null);
            if (localEra.isLocalTime()) {
                since -= (long)n2;
            }
            if (n >= since) {
                date.setLocalEra(localEra);
                date.setLocalYear(date.getNormalizedYear() - localEra.getSinceDate().getYear() + 1);
                break;
            }
            --i;
        }
        if (i < 0) {
            date.setLocalEra(null);
            date.setLocalYear(date.getNormalizedYear());
        }
        date.setNormalized(true);
        return date;
    }
    
    @Override
    public LocalGregorianCalendar.Date newCalendarDate() {
        return new LocalGregorianCalendar.Date();
    }
    
    @Override
    public LocalGregorianCalendar.Date newCalendarDate(TimeZone timeZone) {
        return new Date(timeZone);
    }
    
    @Override
    public boolean validate(CalendarDate calendarDate) {
        LocalGregorianCalendar.Date date = (LocalGregorianCalendar.Date)calendarDate;
        Era era = date.getEra();
        if (era != null) {
            if (!this.validateEra(era)) {
                return false;
            }
            date.setNormalizedYear(era.getSinceDate().getYear() + date.getYear() - 1);
            LocalGregorianCalendar.Date calendarDate2 = this.newCalendarDate(calendarDate.getZone());
            calendarDate2.setEra(era).setDate(calendarDate.getYear(), calendarDate.getMonth(), calendarDate.getDayOfMonth());
            this.normalize(calendarDate2);
            if (calendarDate2.getEra() != era) {
                return false;
            }
        }
        else {
            if (calendarDate.getYear() >= this.eras[0].getSinceDate().getYear()) {
                return false;
            }
            date.setNormalizedYear(date.getYear());
        }
        return super.validate(date);
    }
    
    private boolean validateEra(Era era) {
        int i = 0;
        while (i < this.eras.length) {
            if (era == this.eras[i]) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    @Override
    public boolean normalize(CalendarDate calendarDate) {
        if (calendarDate.isNormalized()) {
            return true;
        }
        this.normalizeYear(calendarDate);
        LocalGregorianCalendar.Date date = (LocalGregorianCalendar.Date)calendarDate;
        super.normalize(date);
        int n = 0;
        long time = 0L;
        int normalizedYear = date.getNormalizedYear();
        Era localEra = null;
        int i;
        for (i = this.eras.length - 1; i >= 0; --i) {
            localEra = this.eras[i];
            if (localEra.isLocalTime()) {
                CalendarDate sinceDate = localEra.getSinceDate();
                int year = sinceDate.getYear();
                if (normalizedYear > year) {
                    break;
                }
                if (normalizedYear == year) {
                    int month = date.getMonth();
                    int month2 = sinceDate.getMonth();
                    if (month > month2) {
                        break;
                    }
                    if (month == month2) {
                        int dayOfMonth = date.getDayOfMonth();
                        int dayOfMonth2 = sinceDate.getDayOfMonth();
                        if (dayOfMonth > dayOfMonth2) {
                            break;
                        }
                        if (dayOfMonth == dayOfMonth2) {
                            if (date.getTimeOfDay() >= sinceDate.getTimeOfDay()) {
                                break;
                            }
                            --i;
                            break;
                        }
                    }
                }
            }
            else {
                if (n == 0) {
                    time = super.getTime(calendarDate);
                    n = 1;
                }
                if (time >= localEra.getSince(calendarDate.getZone())) {
                    break;
                }
            }
        }
        if (i >= 0) {
            date.setLocalEra(localEra);
            date.setLocalYear(date.getNormalizedYear() - localEra.getSinceDate().getYear() + 1);
        }
        else {
            date.setEra(null);
            date.setLocalYear(normalizedYear);
            date.setNormalizedYear(normalizedYear);
        }
        date.setNormalized(true);
        return true;
    }
    
    @Override
    void normalizeMonth(CalendarDate calendarDate) {
        this.normalizeYear(calendarDate);
        super.normalizeMonth(calendarDate);
    }
    
    void normalizeYear(CalendarDate calendarDate) {
        LocalGregorianCalendar.Date date = (LocalGregorianCalendar.Date)calendarDate;
        Era era = date.getEra();
        if (era == null || !this.validateEra(era)) {
            date.setNormalizedYear(date.getYear());
        }
        else {
            date.setNormalizedYear(era.getSinceDate().getYear() + date.getYear() - 1);
        }
    }
    
    public boolean isLeapYear(int n) {
        return CalendarUtils.isGregorianLeapYear(n);
    }
    
    public boolean isLeapYear(Era era, int n) {
        if (era == null) {
            return this.isLeapYear(n);
        }
        return this.isLeapYear(era.getSinceDate().getYear() + n - 1);
    }
    
    @Override
    public void getCalendarDateFromFixedDate(CalendarDate calendarDate, long n) {
        LocalGregorianCalendar.Date date = (LocalGregorianCalendar.Date)calendarDate;
        super.getCalendarDateFromFixedDate(date, n);
        this.adjustYear(date, (n - 719163L) * 86400000L, 0);
    }
    
    public static class Date extends BaseCalendar.Date
    {
        private int gregorianYear;
        
        protected Date() {
            this.gregorianYear = Integer.MIN_VALUE;
        }
        
        protected Date(TimeZone timeZone) {
            super(timeZone);
            this.gregorianYear = Integer.MIN_VALUE;
        }
        
        @Override
        public LocalGregorianCalendar.Date setEra(Era era) {
            if (this.getEra() != era) {
                super.setEra(era);
                this.gregorianYear = Integer.MIN_VALUE;
                return this;
            }
            return this;
        }
        
        @Override
        public LocalGregorianCalendar.Date addYear(int n) {
            super.addYear(n);
            this.gregorianYear += n;
            return this;
        }
        
        @Override
        public LocalGregorianCalendar.Date setYear(int year) {
            if (this.getYear() != year) {
                super.setYear(year);
                this.gregorianYear = Integer.MIN_VALUE;
                return this;
            }
            return this;
        }
        
        @Override
        public int getNormalizedYear() {
            return this.gregorianYear;
        }
        
        @Override
        public void setNormalizedYear(int gregorianYear) {
            this.gregorianYear = gregorianYear;
        }
        
        void setLocalEra(Era era) {
            super.setEra(era);
        }
        
        void setLocalYear(int year) {
            super.setYear(year);
        }
        
        @Override
        public String toString() {
            String string = super.toString();
            String substring = string.substring(string.indexOf(84));
            StringBuffer sb = new StringBuffer();
            Era era = this.getEra();
            if (era != null) {
                String abbreviation = era.getAbbreviation();
                if (abbreviation != null) {
                    sb.append(abbreviation);
                }
            }
            sb.append(this.getYear()).append('.');
            CalendarUtils.sprintf0d(sb, this.getMonth(), 2).append('.');
            CalendarUtils.sprintf0d(sb, this.getDayOfMonth(), 2);
            sb.append(substring);
            return sb.toString();
        }
    }
}
